<?php

/**
 * @file
 * Drush commands for Zentropy.
 * 
 * Borrowed from zentropy
 * @link http://drupal.org/project/zentropy
 */

/**
 * Implements of hook_drush_command().
 */
function zentropy_drush_command() {
  return array(
    'zentropy-new-subtheme' => array(
      'description' => 'Set up a new Zentropy subtheme.',
      'aliases' => array('zns'),
      'arguments' => array(
        'name' => 'A name for your theme.',
        'machine_name' => '(optional)[a-z, 0-9, _] A machine-readable name for your theme.',
      ),
      'options' => _zentropy_build_subtheme_options(),
      'examples' => array(
        'example[1]' => 'drush zns "My theme name"',
        'example[2]' => 'drush zns "My theme name" "my_theme_alias"',
      ) + _zentropy_build_subtheme_options_examples(3),
    ),
    'zentropy-download' => array(
      'description' => 'Download and install optional Zentropy components.',
      'aliases' => array('zdl'),
      'arguments' => array(
        'theme' => 'The machine-readable name for your theme.',
        'name' => 'The name of the component. Supported components are: ' . _zentropy_list_components_string(),
      ),
      'examples' => _zentropy_build_download_examples(),
    ),
  );
}

/**
 * Implements hook_drush_help().
 */
function zentropy_drush_help($section) {
  switch ($section) {
    case 'drush:zentropy-new-subtheme':
      return dt("Set up a new Zentropy subtheme.");
    case 'drush:zentropy-download':
      return dt("Download and install optional Zentropy components.");
  }
}

/**
 * Drush Command.
 * Set up a Zentropy base theme.
 * 
 * @see zentropy_drush_command().
 */
function drush_zentropy_new_subtheme($name = NULL, $machine_name = NULL) {
  if (empty($name)) {
    drush_print(dt("\nWe can't create a subtheme with no name. \nPlease provide a name for the subtheme.\nEx. drush zns [name]\n"));
    return;
  }
  
  $machine_name = !empty($machine_name) ? preg_replace('/[^a-z0-9_]+/', '', strtolower($machine_name)) : preg_replace('/[^a-z0-9_]+/', '', strtolower($name));

  // Get subtheme path.
  $zentropy_path = _zentropy_get_path();
  $subtheme_path = _zentropy_get_subtheme_path($machine_name);
  
  // Make a fresh copy of the original SUBTHEME directory.
  $s = drush_copy_dir("{$zentropy_path}/SUBTHEME/", $subtheme_path);
  if (empty($s)) {
    return;
  }

  // Rename the info file and fill in the theme name.
  drush_op('rename', "{$subtheme_path}/SUBTHEME.info", "{$subtheme_path}/{$machine_name}.info");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/{$machine_name}.info", 'SUBTHEME', "{$machine_name}");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/{$machine_name}.info", 'Zentropy Subtheme', "{$name}");

  // Rename stylesheets and script files and fill in the theme name.
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/README.txt", 'SUBTHEME', "{$machine_name}");
  drush_op('rename', "{$subtheme_path}/css/SUBTHEME.css", "{$subtheme_path}/css/{$machine_name}.css");
  drush_op('rename', "{$subtheme_path}/css/SUBTHEME-maintenance.css", "{$subtheme_path}/css/{$machine_name}-maintenance.css");
  drush_op('rename', "{$subtheme_path}/css/ie/SUBTHEME-ielt8.css", "{$subtheme_path}/css/ie/{$machine_name}-ielt8.css");
  drush_op('rename', "{$subtheme_path}/css/ie/SUBTHEME-ielt9.css", "{$subtheme_path}/css/ie/{$machine_name}-ielt9.css");
  drush_op('rename', "{$subtheme_path}/css/layout/SUBTHEME-320.css", "{$subtheme_path}/css/layout/{$machine_name}-320.css");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/layout/{$machine_name}-320.css", 'Zentropy Subtheme', "{$name}");
  drush_op('rename', "{$subtheme_path}/css/layout/SUBTHEME-480.css", "{$subtheme_path}/css/layout/{$machine_name}-480.css");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/layout/{$machine_name}-480.css", 'Zentropy Subtheme', "{$name}");
  drush_op('rename', "{$subtheme_path}/css/layout/SUBTHEME-768.css", "{$subtheme_path}/css/layout/{$machine_name}-768.css");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/layout/{$machine_name}-768.css", 'Zentropy Subtheme', "{$name}");
  drush_op('rename', "{$subtheme_path}/css/layout/SUBTHEME-992.css", "{$subtheme_path}/css/layout/{$machine_name}-992.css");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/layout/{$machine_name}-992.css", 'Zentropy Subtheme', "{$name}");
  drush_op('rename', "{$subtheme_path}/css/layout/SUBTHEME-1382.css", "{$subtheme_path}/css/layout/{$machine_name}-1382.css");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/layout/{$machine_name}-1382.css", 'Zentropy Subtheme', "{$name}");
  drush_op('rename', "{$subtheme_path}/css/layout/SUBTHEME-hipixel.css", "{$subtheme_path}/css/layout/{$machine_name}-hipixel.css");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/css/layout/{$machine_name}-hipixel.css", 'Zentropy Subtheme', "{$name}");
  drush_op('rename', "{$subtheme_path}/js/SUBTHEME.js", "{$subtheme_path}/js/{$machine_name}.js");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/js/{$machine_name}.js", 'SUBTHEME', "{$machine_name}");
  
  // Rename screenshot.
  drush_op('rename', "{$subtheme_path}/images/SUBTHEME.png", "{$subtheme_path}/images/{$machine_name}.png");
  
  // Fill in theme name & machine_name in template.php.
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/template.php", 'Zentropy Subtheme', "{$name}");
  drush_op('_zentropy_file_str_replace', "{$subtheme_path}/template.php", 'SUBTHEME', "{$machine_name}");
  
  // Notify user of the newly created theme.
  drush_print(dt("\n!name sub-theme was created in !path. Awesome!\n",
    array(
      '!name' => $name,
      '!path' => $subtheme_path,
    )
  ));
}

/**
 * Drush Command.
 * Download and install optional components.
 * 
 * @see zentropy_drush_command().
 */
function drush_zentropy_download($theme = NULL, $component = NULL) {
  // Make sure we have a valid machine_name.
  $theme = preg_replace('/[^a-z0-9_]+/', '', strtolower($theme));
  
  // Check whether the theme exists.
  $list = list_themes(FALSE);

  if (!isset($list[$theme])) {
    drush_die(dt('Please enter a valid theme.'));
  }
  
  // Check whether the theme is a Zentropy subtheme.
  if ($list[$theme]->info['base theme'] !== 'zentropy') {
    drush_die(dt('Please enter a valid Zentropy subtheme.'));
  }
  
  // Check whether component is a valid Zentropy component.
  $components = _zentropy_list_components();
  if (!isset($components[$component])) {
    drush_die(dt('Please enter a valid Zentropy component.'));
  }
  
  // Check to see if Drush Make exists. It is part of Drush core in 5.0-rc2 and up.
  // We use Drush Make to avoid having to re-implement the download logic ourselves.
  // This will only work with Drush 5.0-rc2 or newer.
  // @link http://drupal.org/project/drush
  // @link http://drupal.org/project/drush_make
  if (!function_exists('make_download_factory')) {
    drush_die(dt('Zentropy requires a newer version of Drush. Please update Drush to 5.0-rc2 or newer: http://drupal.org/project/drush'));
  }
  
  // Sweet. Awesomeness ensues.

  // Determine download location.
  $subtheme_path = _zentropy_get_subtheme_path($theme);
  $download_location = $subtheme_path . "/{$components[$component]['target']}/{$component}";
  // Create download array for Drush Make.
  $download = array(
    'type' => $components[$component]['type'],
    'url' => $components[$component]['url'],
  );
  
  $dir = make_download_file($components[$component]['name'], $download, $download_location);
  
  // Replace placeholders in subtheme files.
  if (isset($components[$component]['replace'])) {
    $replacement = str_replace('#SUBTHEME#', $theme, $components[$component]['replace']);
    drush_op('_zentropy_file_str_replace', "{$subtheme_path}/template.php", "#{$component}#", "{$replacement}");
  }
  
  // Display success message.
  drush_print(dt("\nZentropy component: !component successfully downloaded and installed to !dir.", array('!component' => $components[$component]['name'], '!dir' => $dir)));
  
  // Display any additional component instructions.
  if (isset($components[$component]['instructions'])) {
    drush_print(dt("\n" . $components[$component]['instructions']));
  }
}

/**
 * Helper function.
 * Returns available optional Zentropy components.
 * Returns an array where the keys are the components shortname and the values are:
 *  - name: Component full name.
 *  - url: Component download URL.
 *  - type: Component download type ('get', 'file' or 'git').
 *  - target: Component target directory.
 *  - replace: Optional. Component replacement string.
 * 
 * @return array
 */
function _zentropy_list_components() {
  return array(
    'modernizr' => array(
      'name' => 'Modernizr (Development Version)',
      // We use the development version. Please replace this with a Custom Build (http://www.modernizr.com/download/) for production
      'url' => 'http://www.modernizr.com/downloads/modernizr.js',
      'type' => 'get',
      'target' => 'js',
      'replace' => "drupal_add_js(drupal_get_path('theme', '#SUBTHEME#') .'/js/modernizr/modernizr.js', array('group' => CSS_THEME, 'every_page' => TRUE));",
      'instructions' => "Modernizr already includes an HTML5Shiv out of the box. Please go to your subtheme's settings page and disable the 'Use an HTML5 Shiv' option. Failure to do this WILL break things. Also, check out http://www.modernizr.com for full instructions on how to use Modernizr.",
    ),
    'selectivizr' => array(
      'name' => 'Selectivizr',
      'url' => 'http://selectivizr.com/downloads/selectivizr-1.0.2.zip',
      'type' => 'get',
      'target' => 'js',
      'replace' => "drupal_add_js(drupal_get_path('theme', '#SUBTHEME#') .'/js/selectivizr/selectivizr-min.js', array('group' => CSS_THEME, 'every_page' => TRUE));",
    ),
    'pie' => array(
      'name' => 'CSS3Pie',
      'url' => 'http://css3pie.com/download-latest',
      'type' => 'get',
      'target' => 'js',
      'instructions' => 'Check out http://css3pie.com for full instructions on how to use CSS3Pie.',
    ),
    'coolinput' => array(
      'name' => 'jQuery Coolinput',
      'url' => 'https://github.com/downloads/alexweber/jquery.coolinput/jquery.coolinput-2.0.tar.gz',
      'type' => 'get',
      'target' => 'js',
      'replace' => "drupal_add_js(drupal_get_path('theme', '#SUBTHEME#') .'/js/coolinput/jquery.coolinput.min.js', array('group' => CSS_THEME, 'every_page' => TRUE));",
      'instructions' => 'Check out https://github.com/alexweber/jquery.coolinput for instructions on how to use Coolinput.'
    ),
  );
}

/**
 * Helper function.
 * Returns available optional Zentropy components as a formatted string.
 * 
 * @return array
 */
function _zentropy_list_components_string() {
  $components = array();
  
  foreach (_zentropy_list_components() as $component) {
    $components[] = $component['name'];
  }
  
  return implode(', ', $components);
}

/**
 * Helper function.
 * Returns zentropy-new-subtheme option list for optional Zentropy components.
 * 
 * @return array
 */
function _zentropy_build_subtheme_options() {
  $options = array();
  
  foreach (_zentropy_list_components() as $key => $val) {
    $options["with-{$key}"] = "Include {$val['name']} in the new subtheme.";
  }
  
  return $options;
}

/**
 * Helper function.
 * Builds additional examples for zentropy-new-subtheme command.
 * 
 * @param int $i
 * @return array
 */
function _zentropy_build_subtheme_options_examples($i) {
  $components = array_keys(_zentropy_list_components());
  $j = $i + 1;
  
  // Add 2 new random examples.
  $temp = array_rand($components);
  $component1 = $components[$temp];
  // Remove component from list to avoid duplication.
  unset($components[$temp]);
  $component2 = $components[array_rand($components)];
  
  return array(
    "example[{$i}]" => 'drush zns "My theme name" "my_theme_alias" --with-' . $component1,
    "example[{$j}]" => 'drush zns "My theme name" "my_theme_alias" --with-' . $component1 . ' --with-' . $component2,
  );
}

/**
 * Helper function.
 * Builds examples for zentropy-download command.
 * 
 * @return array
 */
function _zentropy_build_download_examples() {
  $examples = array();
  $i = 1;
  
  foreach (array_keys(_zentropy_list_components()) as $key) {
    $examples["example[{$i}]"] = "drush zdl zentropy {$key}";
    ++$i;
  }
  
  return $examples;
}

/**
 * Helper function.
 * Get path to Zentropy theme.
 * 
 * @return string
 */
function _zentropy_get_path() {
  return drush_locate_root() . '/' . drupal_get_path('theme', 'zentropy');
}

/**
 * Helper function.
 * Get path to a Zentropy subtheme.
 * 
 * @param string $machine_name
 * @return string
 */
function _zentropy_get_subtheme_path($machine_name) {
  $zentropy_path = _zentropy_get_path();
  // From Zentropy's location, we move up one directory and construct the path where our subtheme will be created.
  $_zentropy_path = explode('/', $zentropy_path);
  array_pop($_zentropy_path);
  $theme_dir = implode('/', $_zentropy_path);
  return $theme_dir . '/' . $machine_name;
}

/**
 * Helper function.
 * Replace strings in a file.
 * 
 * @param string @file_path
 * @param string $find
 * @param string $replace
 * @see drush_zentropy_new_subtheme().
 */
function _zentropy_file_str_replace($file_path, $find, $replace) {
  $file_contents = file_get_contents($file_path);
  $file_contents = str_replace($find, $replace, $file_contents);
  file_put_contents($file_path, $file_contents);
}